import NavigateEvent from './NavigateEvent';
import NavigationHistoryEntry, { setNavigationHistoryEntryState, } from './NavigationHistoryEntry';
import NavigationHistoryStack from './NavigationHistoryStack';
import { NAVIGATION_STATE_KEY, NAVIGATION_HISTORY_ENTRY_STATE_KEY, } from './constants';
import { createKey } from './utils';
const nativePushState = History.prototype.pushState;
const nativeReplaceState = History.prototype.replaceState;
function createNavigationHistoryState(state, navigation, entry) {
    return Object.assign({}, state, {
        [NAVIGATION_STATE_KEY]: navigation.key,
        [NAVIGATION_HISTORY_ENTRY_STATE_KEY]: entry.key,
    });
}
function getNavigationCurrentEntryKey() {
    var _a;
    return (_a = history.state) === null || _a === void 0 ? void 0 : _a[NAVIGATION_HISTORY_ENTRY_STATE_KEY];
}
function getNavigationEntry(entries, entryKey = getNavigationCurrentEntryKey()) {
    let matchedEntry = entries[entries.length - 1];
    for (let index = 0; index < entries.length; index++) {
        const entry = entries[index];
        if (entry.key === entryKey) {
            matchedEntry = entry;
        }
    }
    return matchedEntry;
}
class Navigation extends EventTarget {
    constructor() {
        var _a;
        super();
        const cacheKey = (_a = history.state) === null || _a === void 0 ? void 0 : _a[NAVIGATION_STATE_KEY];
        const key = cacheKey !== null && cacheKey !== void 0 ? cacheKey : createKey();
        const shouldInit = cacheKey !== key;
        this.key = key;
        this.entryStack = new NavigationHistoryStack(this);
        // init
        if (shouldInit) {
            const entry = new NavigationHistoryEntry(0, location.pathname + location.search + location.hash);
            nativeReplaceState.call(history, createNavigationHistoryState(history.state, this, entry), '', location.pathname + location.search + location.hash);
            this.entryStack.push(entry);
        }
        // currentEntry
        this.currentEntry = getNavigationEntry(this.entryStack.entries());
        const that = this;
        // rewrite
        History.prototype.pushState = function (data, unused, url) {
            const entry = new NavigationHistoryEntry(that.currentEntry.index + 1, url);
            const navigateEvent = new NavigateEvent('navigate', {
                navigationType: 'push',
                destination: entry,
            });
            that.dispatchEvent(navigateEvent);
            if (!navigateEvent.defaultPrevented) {
                const state = createNavigationHistoryState(data, that, entry);
                nativePushState.call(this, state, unused, url);
                that.entryStack.push(entry);
                that.currentEntry = entry;
            }
        };
        History.prototype.replaceState = function (data, unused, url) {
            const entry = new NavigationHistoryEntry(that.currentEntry.index, url, navigation.currentEntry.key);
            const navigateEvent = new NavigateEvent('navigate', {
                navigationType: 'replace',
                destination: entry,
            });
            that.dispatchEvent(navigateEvent);
            if (!navigateEvent.defaultPrevented) {
                const state = createNavigationHistoryState(data, that, entry);
                nativeReplaceState.call(this, state, unused, url);
                that.entryStack.replace(entry);
                that.currentEntry = entry;
            }
        };
        window.addEventListener('popstate', function (event) {
            const entryKey = event.state[NAVIGATION_HISTORY_ENTRY_STATE_KEY];
            const entry = getNavigationEntry(that.entries(), entryKey);
            that.currentEntry = entry;
            const navigateEvent = new NavigateEvent('navigate', {
                navigationType: 'traverse',
                destination: entry,
                cancelable: false,
            });
            that.dispatchEvent(navigateEvent);
        });
    }
    updateCurrentEntry(options) {
        setNavigationHistoryEntryState(this.currentEntry.key, options.state);
    }
    entries() {
        return this.entryStack.entries();
    }
}
const navigation = new Navigation();
export { navigation };
export default Navigation;
