"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.setNavigationHistoryEntryState = exports.getNavigationHistoryEntryState = exports.getNavigationHitoryEntryStateCacheKey = void 0;
const constants_1 = require("./constants");
const utils_1 = require("./utils");
function getNavigationHitoryEntryStateCacheKey(key) {
    return `${constants_1.NAVIGATION_HISTORY_ENTRY_STATE_CACHE_KEY}_${key}`;
}
exports.getNavigationHitoryEntryStateCacheKey = getNavigationHitoryEntryStateCacheKey;
function getNavigationHistoryEntryState(key) {
    const stateCacheKey = getNavigationHitoryEntryStateCacheKey(key);
    const stateCache = sessionStorage.getItem(stateCacheKey);
    if (stateCache) {
        try {
            return JSON.parse(stateCache);
        }
        catch (error) {
            // ignore
        }
    }
    return undefined;
}
exports.getNavigationHistoryEntryState = getNavigationHistoryEntryState;
function setNavigationHistoryEntryState(key, state) {
    const stateCacheKey = getNavigationHitoryEntryStateCacheKey(key);
    if (state === null || state === undefined) {
        sessionStorage.removeItem(stateCacheKey);
    }
    else {
        const stateCache = JSON.stringify(state);
        sessionStorage.setItem(stateCacheKey, stateCache);
    }
}
exports.setNavigationHistoryEntryState = setNavigationHistoryEntryState;
class NavigationHistoryEntry {
    constructor(index, url, key, id) {
        this.index = index;
        this.url = url;
        this.key = key !== null && key !== void 0 ? key : (0, utils_1.createKey)();
        this.id = id !== null && id !== void 0 ? id : (0, utils_1.createKey)();
    }
    toJSON() {
        return {
            index: this.index,
            url: this.url,
            key: this.key,
            id: this.id,
        };
    }
    getState() {
        return getNavigationHistoryEntryState(this.key);
    }
}
exports.default = NavigationHistoryEntry;
