"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.getNavigationHistoryStackCacheKey = void 0;
const constants_1 = require("./constants");
const NavigationHistoryEntry_1 = require("./NavigationHistoryEntry");
function getNavigationHistoryStackCacheKey(navigation) {
    return `${constants_1.NAVIGATION_HISTORY_STACK_CACHE_KEY}_${navigation.key}`;
}
exports.getNavigationHistoryStackCacheKey = getNavigationHistoryStackCacheKey;
class NavigationHistoryStack {
    constructor(navigation) {
        this.navigation = navigation;
        let stack = [];
        try {
            stack = JSON.parse(sessionStorage.getItem(getNavigationHistoryStackCacheKey(navigation)) || '[]').map((entryCache, index) => {
                return new NavigationHistoryEntry_1.default(entryCache.index, entryCache.url, entryCache.key, entryCache.id);
            });
        }
        catch (error) {
            stack = [];
        }
        this.stack = stack;
    }
    entries() {
        return this.stack;
    }
    push(entry) {
        try {
            this.stack = this.stack.slice(0, entry.index);
            this.stack.push(entry);
            sessionStorage.setItem(getNavigationHistoryStackCacheKey(this.navigation), JSON.stringify(this.stack));
        }
        catch (error) {
            // TODO: overflow
        }
    }
    replace(entry) {
        this.stack.splice(entry.index, 1, entry);
        try {
            sessionStorage.setItem(getNavigationHistoryStackCacheKey(this.navigation), JSON.stringify(this.stack));
        }
        catch (error) {
            // TODO: overflow
        }
    }
}
exports.default = NavigationHistoryStack;
